/*
 *	emu.c
 *
 *	SUPERVISION/EMU
 *
 *	* Fri Apr 15 05:00:00 JST 2005 Naoyuki Sawa
 *	- 쐬JnB
 */
#include "app.h"

/****************************************************************************
 *	
 ****************************************************************************/

int
sv_init(const char* fname)
{
	FILE* fp;
	int size;

	/* ROMC[W[h܂B */
	fp = fopen(fname, "rb");
	if(!fp) {
		return -1;
	}
	size = fread(rom, 1, 0x10000, fp);
	fclose(fp);

	/* ROMC[WTCYɂ... */
	switch(size) {
	case  0x8000: /* 32KB */
		/* O32KB㔼32KBփ~[܂B */
		memcpy(&rom[0x8000], &rom[0x0000], 0x8000);
		break;
	case 0x10000: /* 64KB */
		/** no job **/
		break;
	default:
		return -1;
	}

	/* oNݒ肵܂B */
	rom_bank = &rom[0] - 0x8000/*ROM window base address*/;

	/* CA^C}NA܂B */
	memset(ram, 0, sizeof ram);
	TIMER = 0;

	/* Video/SoundZbg܂B */
	svvideo_reset(ram);
	svsound_reset(rom);

	/* CPUZbg܂B */
	w65c02_reset();

	return 0;
}

void
input_update()
{
	int pad = pcePadGetDirect();
	int input = 0xff;
	if(pad & PAD_RI    ) input &= ~(1<<0);
	if(pad & PAD_LF    ) input &= ~(1<<1);
	if(pad & PAD_DN    ) input &= ~(1<<2);
	if(pad & PAD_UP    ) input &= ~(1<<3);
	if(pad & PAD_B     ) input &= ~(1<<4);
	if(pad & PAD_A     ) input &= ~(1<<5);
	if(pad & PAD_SELECT) input &= ~(1<<6);
	if(pad & PAD_START ) input &= ~(1<<7);
	ram[0x2020/*input read*/] = input;
}

int
sv_stream_callback(short* wbuff, int param)
{
///	svsound_mix(wbuff);
///	return 0;
return -1; /*TODO: */
}

int
sv_run(const char* fname, int sound_on)
{
	int cnt = 0;
	//
	int line;
	unsigned char* draw;

	if(sv_init(fname) != 0) {
		return -1;
	}

	if(sound_on) {
		stream_play(SVSOUNDBUFLEN, sv_stream_callback, 0, 0);
	}

	cnt = 0;
	for(;;) {
		/* ͍XVB */
		input_update();

		/* t[XLbvB */
		draw = NULL;
		if((cnt += FPS) >= 60) {
			cnt -= 60;
			draw = vbuff;
		}

		line = 160;
		do {
			w65c02_run(SUPERVISION_CLOCK/60/160);
			if(TIMER) {
				TIMER -= SUPERVISION_CLOCK/60/160;
				if(TIMER <= 0) {
					TIMER = 0;
					ram[0x2024] |= (1<<0); /* Timer}[N(?) */
					ram[0x2027] |= (1<<0); /* IRQv|=Timer (?) */
					w65c02_irq();
				}
			}
		} while(--line);
		if(ram[0x2026] & 1) { /* NMI enable? */
			w65c02_nmi();
		}

		/* ʍXVB */
		if(draw) {
			svvideo_update(vbuff);
			svvideo_reduce(vbuff, vbuff);
			schedule();
			if((joy & PAD_START) && (joy & PAD_SELECT)) {
				break;
			}
		}
	}

	if(sound_on) {
		stream_stop();
	}

	return 0;
}

int
reg_read(int addr)
{
	int data;

	switch(addr) {
	/*========== Video ==========*/
	case 0x2000:
	case 0x2001:
	case 0x2002:
	case 0x2003:
		return svvideo_read(addr);
	/*========== Sound ==========*/
	case 0x2010:
	case 0x2011:
	case 0x2012:
	case 0x2014:
	case 0x2015:
	case 0x2016:
	case 0x2018:
	case 0x2019:
	case 0x201a:
	case 0x201b:
	case 0x201c:
	case 0x2028:
	case 0x2029:
	case 0x202a:
		return svsound_read(addr);
	/*========== IRQv(?) ==========*/
	case 0x2027:
		/* IRQv\tOłB
		 * - bit0TimerŁAbit1SoundDMA݂łB
		 *   ȂƂbit0̋T|[gȂƁACrystball܂B
		 * - ̃G~[^́AIRQv̎NAsĂ܂񂪁A
		 *   vO̓삩琄肷ɁAǂݏoNAƎv܂B
		 *   vOAI0ޏsĂȂłB
		 */
		data = ram[0x2027];
		ram[0x2027] = 0;
		return data;
	default:
		return ram[addr];
	}
}

void
reg_write(int addr, int data)
{
	ram[addr] = data;

	switch(addr) {
	/*========== Video ==========*/
	case 0x2000:
	case 0x2001:
	case 0x2002:
	case 0x2003:
		svvideo_write(addr, data);
		return;
	/*========== Sound ==========*/
	case 0x2010:
	case 0x2011:
	case 0x2012:
	case 0x2014:
	case 0x2015:
	case 0x2016:
	case 0x2018:
	case 0x2019:
	case 0x201a:
	case 0x201b:
	case 0x201c:
	case 0x2028:
	case 0x2029:
	case 0x202a:
		svsound_write(addr, data);
		return;
	/*========== I/O ==========*/
	case 0x2022:
		ram[0x2021] = data; /* $2022(W) -> $2021(R) (?) */
		return;
	/*========== Timer ==========*/
	case 0x2023:
		TIMER = data << 8;
		ram[0x2024] &= ~(1<<0); /* Timer}[N(?) */
		return;
	/*========== Bank,NMI ==========*/
	case 0x2026:
		/* Bank sel          ?bb????n
		 *                    ||    |
		 *          ++--------++    +-- NMI enable
		 *          ||
		 * ROM addr bb000000 00000000
		 */
		rom_bank = &rom[(data & 0x60) << 9] - 0x8000/*ROM window base address*/;
		return;
	}
}
